/*  Copyright (C) 2012-2022 by László Nagy
    This file is part of Bear.

    Bear is a tool to generate compilation database for clang tooling.

    Bear is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bear is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include "Domain.h"
#include "libmain/Application.h"
#include "libmain/ApplicationLogConfig.h"
#include "libflags/Flags.h"
#include "libresult/Result.h"
#include "libsys/Process.h"

namespace wr {
    using namespace domain;
    using namespace sys;

    struct Command : ps::Command {
        Command(wr::SessionLocator session, wr::Execution execution) noexcept;

        [[nodiscard]] rust::Result<int> execute() const override;

        NON_DEFAULT_CONSTRUCTABLE(Command)
        NON_COPYABLE_NOR_MOVABLE(Command)

    protected:
        wr::SessionLocator session_;
        wr::Execution execution_;
    };

    struct Application : ps::Application {
        Application() noexcept;
        rust::Result<ps::CommandPtr> command(int argc, const char **argv, const char **envp) const override;

        static rust::Result<ps::CommandPtr> from_envs(int argc, const char **argv, const char **envp);
        static rust::Result<ps::CommandPtr> from_args(const flags::Arguments &args, const char **envp);
        static rust::Result<flags::Arguments> parse(int argc, const char **argv) ;

    private:
        ps::ApplicationLogConfig const &log_config;
    };


    class WrapperBuilder : public Process::Builder {
    public:
        explicit WrapperBuilder(fs::path program, const std::map<std::string, std::string>&);
        ~WrapperBuilder() = default;

        WrapperBuilder& get_options_from_environment();

        template <typename InputIt>
        WrapperBuilder& add_arguments(InputIt first, InputIt last)
        {
            for (InputIt it = first; it != last; ++it) {
                add_argument(*it);
            }
            add_arguments_new();
            parameters_ = parameters_replace_;
            return *this;
        }

        WrapperBuilder& add_arguments_new();

        std::list<std::string> split_optons(std::string& options);

        Execution get_new_execution(Execution& execution);
        bool enableWrapperBuilder();
    public:
        NON_DEFAULT_CONSTRUCTABLE(WrapperBuilder)

    private:
        std::list<std::string> compile_flags_;
        std::list<std::string> ld_flags_;
        std::list<std::string> parameters_replace_; 
    };
}
